@@ -1,19 +1,21 @@
 # Schedwi
 #   Copyright (C) 2007 Herve Quatremain
-#     
-#   This program is free software; you can redistribute it and/or modify
+#
+#   This file is part of Schedwi.
+#
+#   Schedwi is free software; you can redistribute it and/or modify
 #   it under the terms of the GNU General Public License as published by
-#   the Free Software Foundation; either version 2 of the License, or
+#   the Free Software Foundation; either version 3 of the License, or
 #   (at your option) any later version.
-#     
-#   This program is distributed in the hope that it will be useful,
+#
+#   Schedwi is distributed in the hope that it will be useful,
 #   but WITHOUT ANY WARRANTY; without even the implied warranty of
 #   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-#   GNU Library General Public License for more details.
+#   GNU General Public License for more details.
 #
 #   You should have received a copy of the GNU General Public License
-#   along with this program; if not, write to the Free Software
-#   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+#   along with this program.  If not, see <http://www.gnu.org/licenses/>.
+#
 
 # MySql tables definitions for the Schedwi server
 #
@@ -90,7 +92,7 @@ CREATE TABLE job_start_limit (
 
 
 #
-# Max number of minutes of the job/jobset to complete once running.  0 means
+# Max number of minutes for the job/jobset to complete once running.  0 means
 # no limit
 #
 CREATE TABLE job_max_duration (
@@ -101,6 +103,28 @@ CREATE TABLE job_max_duration (
 );
 
 
+#
+# Number of retries when the job/jobset fail
+#
+CREATE TABLE job_retries (
+ job_id         bigint UNSIGNED DEFAULT 0 NOT NULL,
+ retries        smallint DEFAULT 0 NOT NULL,
+
+ PRIMARY KEY (job_id)
+);
+
+
+#
+# Number of minutes between retries
+#
+CREATE TABLE job_retries_interval (
+ job_id           bigint UNSIGNED DEFAULT 0 NOT NULL,
+ retries_interval smallint DEFAULT 0 NOT NULL,
+
+ PRIMARY KEY (job_id)
+);
+
+
 #
 # Username of the account under which the job command will be run
 # on the remote host
@@ -226,7 +250,7 @@ CREATE TABLE environment_var (
 # Associate environments with jobs.  A position of -1 means that the job
 # does not have any environment.
 # The final job environment is the concatenation of all its parents 
-# environments plus its associated host environement (see
+# environments plus its associated host environment (see
 # table host_environment)
 #
 CREATE TABLE job_environment (
@@ -588,6 +612,30 @@ CREATE TABLE job_max_duration_s (
 );
 
 
+#
+# Number of retries when the job/jobset fail
+#
+CREATE TABLE job_retries_s (
+ job_id         bigint UNSIGNED DEFAULT 0 NOT NULL,
+ retries        smallint DEFAULT 0 NOT NULL,
+ workload_date  int DEFAULT 0 NOT NULL,
+
+ PRIMARY KEY (workload_date,job_id)
+);
+
+
+#
+# Number of minutes between retries
+#
+CREATE TABLE job_retries_interval_s (
+ job_id           bigint UNSIGNED DEFAULT 0 NOT NULL,
+ retries_interval smallint DEFAULT 0 NOT NULL,
+ workload_date  int DEFAULT 0 NOT NULL,
+
+ PRIMARY KEY (workload_date,job_id)
+);
+
+
 #
 # Username of the account under which the job command will be run
 # on the remote host
@@ -722,7 +770,7 @@ CREATE TABLE environment_var_s (
 # Associate environments with jobs.  A position of -1 means that the job
 # does not have any environment.
 # The final job environment is the concatenation of all its parents 
-# environments plus its associated host environement (see
+# environments plus its associated host environment (see
 # table host_environment_s)
 #
 CREATE TABLE job_environment_s (
@@ -818,6 +866,7 @@ CREATE TABLE job_status (
  job_id         bigint UNSIGNED DEFAULT 0 NOT NULL,
  status         tinyint DEFAULT 1 NOT NULL,
  start_time     bigint UNSIGNED DEFAULT 0 NOT NULL,
+ retry_num      smallint UNSIGNED DEFAULT 0 NOT NULL,
  error_msg      varchar(255) DEFAULT '' NOT NULL,
 
  PRIMARY KEY (workload_date,job_id)
@@ -1007,6 +1056,8 @@ CREATE TABLE link_s (
 INSERT INTO job_main VALUES (1,0,"Root",0,1,"Root Jobset",100,100,4,0);
 INSERT INTO job_start_limit VALUES (1, 1440);
 INSERT INTO job_max_duration VALUES (1, 0);
+INSERT INTO job_retries VALUES (1, 0);
+INSERT INTO job_retries_interval VALUES (1, 5);
 INSERT INTO job_username VALUES (1,"nobody");
 INSERT INTO job_file_out VALUES (1,"/dev/null");
 INSERT INTO job_file_err VALUES (1,"/dev/null");
@@ -1015,8 +1066,8 @@ INSERT INTO job_success_return_code VALUES (1,0);
 INSERT INTO job_arguments VALUES (1,-1,"");
 INSERT INTO environments VALUES (1,"Default","Default environment");
 INSERT INTO job_environment VALUES (1,1,-1);
-INSERT INTO hosts (id,hostname,description)
-       VALUES (1,"localhost","Local client");
+INSERT INTO hosts (id,hostname,description,sslenable)
+       VALUES (1,"localhost","Local client",0);
 INSERT INTO job_host VALUES (1,1);
 INSERT INTO jobset_background VALUES (1,"",2,"#678cc2","#d7e1ef",800,600);
 INSERT INTO calendars (id,parent,name,entry_type,formula) VALUES
@@ -2215,7 +2266,7 @@ INSERT INTO calendars (id,parent,name,entry_type,formula) VALUES
 (1193,1178,"United Nations' Day",0,"10/24"),
 (1194,1178,"All Saints' Day",0,"11/1"),
 (1195,1178,"All Souls' Day",0,"11/2"),
-(1196,1178,"Vertieres Battle Day",0,"18/11"),
+(1196,1178,"Vertieres Battle Day",0,"11/18"),
 (1197,1178,"Christmas Day",0,"12/25"),
 (1198,18,"El Salvador",1,""),
 (1199,1198,"New Year's Day",0,"1/1"),
@@ -2262,6 +2313,6 @@ INSERT INTO calendars (id,parent,name,entry_type,formula) VALUES
 #
 source schedwi_binary_data.sql
 
-INSERT INTO schedwi_version VALUES ("0.1.0",NOW());
+INSERT INTO schedwi_version VALUES ("0.1.1",NOW());
 
 #******************************** End Of File ********************************
