@@ -1,20 +1,20 @@
 /* Schedwi
    Copyright (C) 2007 Herve Quatremain
 
-   This program is free software; you can redistribute it and/or modify
+   This file is part of Schedwi.
+
+   Schedwi is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
+   the Free Software Foundation; either version 3 of the License, or
    (at your option) any later version.
 
-   This program is distributed in the hope that it will be useful,
+   Schedwi is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU Library General Public License for more details.
+   GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
-   USA.
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 /*
@@ -263,7 +263,7 @@ net_server_ssl (gnutls_session_t *session, int fd,
 		return -1;
 	}
 
-	crt.data = remote_crt;
+	crt.data = (unsigned char *)remote_crt;
 	crt.size = remote_crt_len;
 	ret = gnutls_certificate_set_x509_trust_mem (*trusted_cert_ptr, &crt,
 							GNUTLS_X509_FMT_PEM);
@@ -447,7 +447,9 @@ _("SSL: Failed to generate the parameters for the RSA algorithms: %s"),
  *  -1 --> Error (a message has been logged by lwc_writeLog())
  */
 int
-net_init_ssl (const char *schedwisrv_crt_file, const char *schedwisrv_key_file)
+net_init_ssl (	const char *schedwisrv_crt_file,
+		const char *schedwisrv_key_file,
+		char quick_random)
 {
 	int ret;
 	gnutls_certificate_credentials_t x509_cred;
@@ -459,6 +461,11 @@ net_init_ssl (const char *schedwisrv_crt_file, const char *schedwisrv_key_file)
 	/* For the support of threads in libgcrypt */
 	gcry_control (GCRYCTL_SET_THREAD_CBS, &gcry_threads_pthread);
 
+	/* Fast random mode (less secure) */
+	if (quick_random != 0) {
+		gcry_control (GCRYCTL_ENABLE_QUICK_RANDOM, 0);
+	}
+
 	/* GNUTLS init */
 	ret = gnutls_global_init ();
 	if (ret != 0) {
@@ -730,7 +737,7 @@ net_client_ssl (const char *port_number, const char *hostname,
 		return -1;
 	}
 
-	crt.data = remote_crt;
+	crt.data = (unsigned char *)remote_crt;
 	crt.size = remote_crt_len;
 	ret = gnutls_certificate_set_x509_trust_mem (*trusted_cert_ptr, &crt,
 							GNUTLS_X509_FMT_PEM);
