@@ -1,19 +1,20 @@
 /* Schedwi
    Copyright (C) 2007 Herve Quatremain
 
-   This program is free software; you can redistribute it and/or modify
+   This file is part of Schedwi.
+
+   Schedwi is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
+   the Free Software Foundation; either version 3 of the License, or
    (at your option) any later version.
 
-   This program is distributed in the hope that it will be useful,
+   Schedwi is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU Library General Public License for more details.
+   GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 /* schedwi_jobtree.c -- Function to manage the job/jobset tree */
@@ -55,13 +56,15 @@ extern int errno;
 #include <stopjob.h>
 #include <testfile.h>
 #include <lib_functions.h>
+#include <module.h>
 #include <schedwi_jobtree_priv.h>
 
 #define TABSTOP 4
 
 /*
  * Error callback function for the sql_job_get_start_limit(),
- * sql_job_get_max_duration() and sql_status_get () functions
+ * sql_job_get_max_duration(), sql_job_get_retries(),
+ * sql_job_get_retries_interval() and sql_status_get () functions
  */
 static void
 sql_get_error_logger (void *data, const char *msg, unsigned int err_code)
@@ -86,6 +89,7 @@ sql_get_error_logger (void *data, const char *msg, unsigned int err_code)
  */
 static schedwi_jobtree_node_ptr
 new_schedwi_jobtree_node (	const char *id,
+				char *path,
 				int workload_date,
 				const char *node_type,
 				schedwi_date job_date,
@@ -101,7 +105,7 @@ new_schedwi_jobtree_node (	const char *id,
 	int n, i, j;
 
 #if HAVE_ASSERT_H
-	assert (   id != NULL && node_type != NULL
+	assert (   id != NULL && path != NULL && node_type != NULL
 		&& job_time != NULL && hierarchy_list != NULL);
 #endif
 
@@ -131,8 +135,27 @@ new_schedwi_jobtree_node (	const char *id,
 	}
 	ptr->max_duration = time_limit * 60; /* The limit in DB is in min */
 
+	if (sql_job_get_retries (	workload_date, hierarchy_list,
+					&(ptr->retries),
+					sql_get_error_logger, NULL) != 0)
+	{
+		free (ptr);
+		return NULL;
+	}
+
+	if (sql_job_get_retries_interval (workload_date, hierarchy_list,
+					&time_limit,
+					sql_get_error_logger, NULL) != 0)
+	{
+		free (ptr);
+		return NULL;
+	}
+	ptr->retries_interval = time_limit * 60; /* The value in DB is in min*/
+
  	if (sql_status_get (	workload_date,
-				id, &current_status, &start_time, NULL,
+				id, &current_status, &start_time,
+				&(ptr->retry_num),
+				NULL,
 				sql_get_error_logger, NULL) != 0)
 	{
 		free (ptr);
@@ -209,6 +232,7 @@ new_schedwi_jobtree_node (	const char *id,
 	ptr->status = (current_status == 0)
 				? JOB_STATUS_STATE_WAITING
 				: job_status_state_int2status (current_status);
+	ptr->path       = path;
 	ptr->parent     = NULL;
 	ptr->children   = NULL;
 	ptr->next       = NULL;
@@ -251,6 +275,9 @@ free_schedwi_jobtree_node (schedwi_jobtree_node_ptr ptr)
 	int i;
 
 	if (ptr != NULL) {
+		if (ptr->path != NULL) {
+			free (ptr->path);
+		}
 		if (ptr->constraint_files != NULL) {
 			i = ptr->num_constraint_files;
 			while (--i >= 0) {
@@ -424,6 +451,8 @@ sql_list_error_logger (void *data, const char *msg, unsigned int err_code)
 static int
 build_jobtree_recur (	schedwi_jobtree_node_ptr jobset,
 			const char *jobset_id,
+			const char *jobset_path,
+			unsigned int jobset_path_length,
 			schedwi_date jobset_date,
 			int workload_date,
 			const char *jobset_time,
@@ -434,22 +463,21 @@ build_jobtree_recur (	schedwi_jobtree_node_ptr jobset,
 {
 	lwc_LL *job_list;
 	char **row;
-	unsigned int sql_ret;
+	unsigned int sql_ret, length;
 	int ret;
-	char *calendars_for_today, *calendars_for_tomorrow;
+	char *calendars_for_today, *calendars_for_tomorrow, *path;
 	schedwi_date date_tomorrow;
 	schedwi_jobtree_node_ptr ptr, nxt;
 
 #if HAVE_ASSERT_H
-	assert (   jobset_id != NULL && calendars != NULL
-		&& hierarchy_list != NULL && jobset_time != NULL
-		&& children != NULL && nb_jobs != NULL);
+	assert (   jobset_id != NULL && jobset_path != NULL
+		&& calendars != NULL && hierarchy_list != NULL
+		&& jobset_time != NULL && children != NULL
+		&& nb_jobs != NULL);
 #endif
 
 	ptr = nxt = NULL;
 
-
-
 	/*
 	 * Build the tree with the jobs and jobsets that must run today
 	 */
@@ -478,10 +506,27 @@ build_jobtree_recur (	schedwi_jobtree_node_ptr jobset,
 		 * row[0] --> ID
 		 * row[1] --> Type (0: jobset and 1: job)
 		 * row[2] --> Start time
+		 * row[3] --> Job/Jobset name
 		 */
 
+		/* Build the full path name of the job/jobset */
+		length = jobset_path_length + schedwi_strlen (row[3]) + 1; 
+		path = (char *) malloc (length + 1);
+		if (path == NULL) {
+			sql_free_row (row);
+			lwc_delLL (	job_list,
+					(void (*)(const void *)) sql_free_row);
+			free_jobtree_all_nodes (nxt);
+			lwc_writeLog (LOG_CRIT, _("Memory allocation error"));
+			return -1;
+		}
+		strcpy (path, jobset_path);
+		path[jobset_path_length] = '/';
+		strcpy (path + jobset_path_length + 1, row[3]);
+
 		/* Add the new job/jobset ID to the hierarchy list */
 		if (lwc_addStartLL (hierarchy_list, row[0]) != 0) {
+			free (path);
 			sql_free_row (row);
 			lwc_delLL (	job_list,
 					(void (*)(const void *)) sql_free_row);
@@ -491,12 +536,14 @@ build_jobtree_recur (	schedwi_jobtree_node_ptr jobset,
 		}
 
 		/* Create the new job object */
-		ptr = new_schedwi_jobtree_node (	row[0], workload_date,
+		ptr = new_schedwi_jobtree_node (	row[0], path,
+							workload_date,
 							row[1],
 							jobset_date, row[2],
 							hierarchy_list);
 		if (ptr == NULL) {
 			lwc_delStartLL (hierarchy_list);
+			free (path);
 			sql_free_row (row);
 			lwc_delLL (	job_list,
 					(void (*)(const void *)) sql_free_row);
@@ -517,6 +564,7 @@ build_jobtree_recur (	schedwi_jobtree_node_ptr jobset,
 		if (ptr->node_type == 0) {
 			ret = build_jobtree_recur (	ptr,
 							row[0],
+							path, length,
 							jobset_date,
 							workload_date,
 							row[2],
@@ -573,10 +621,27 @@ build_jobtree_recur (	schedwi_jobtree_node_ptr jobset,
 		 * row[0] --> ID
 		 * row[1] --> Type (0: jobset and 1: job)
 		 * row[2] --> Start time
+		 * row[3] --> Job/Jobset name
 		 */
 
+		/* Build the full path name of the job/jobset */
+		length = jobset_path_length + schedwi_strlen (row[3]) + 1; 
+		path = (char *) malloc (length + 1);
+		if (path == NULL) {
+			sql_free_row (row);
+			lwc_delLL (	job_list,
+					(void (*)(const void *)) sql_free_row);
+			free_jobtree_all_nodes (nxt);
+			lwc_writeLog (LOG_CRIT, _("Memory allocation error"));
+			return -1;
+		}
+		strcpy (path, jobset_path);
+		path[jobset_path_length] = '/';
+		strcpy (path + jobset_path_length + 1, row[3]);
+
 		/* Add the new job/jobset ID to the hierarchy list */
 		if (lwc_addStartLL (hierarchy_list, row[0]) != 0) {
+			free (path);
 			sql_free_row (row);
 			lwc_delLL (	job_list,
 					(void (*)(const void *)) sql_free_row);
@@ -586,12 +651,14 @@ build_jobtree_recur (	schedwi_jobtree_node_ptr jobset,
 		}
 
 		/* Create the new job object */
-		ptr = new_schedwi_jobtree_node (	row[0], workload_date,
+		ptr = new_schedwi_jobtree_node (	row[0], path,
+							workload_date,
 							row[1],
 							date_tomorrow, row[2],
 							hierarchy_list);
 		if (ptr == NULL) {
 			lwc_delStartLL (hierarchy_list);
+			free (path);
 			sql_free_row (row);
 			lwc_delLL (	job_list,
 					(void (*)(const void *)) sql_free_row);
@@ -612,6 +679,7 @@ build_jobtree_recur (	schedwi_jobtree_node_ptr jobset,
 		if (ptr->node_type == 0) {
 			ret = build_jobtree_recur (	ptr,
 							row[0],
+							path, length,
 							date_tomorrow,
 							workload_date,
 							row[2],
@@ -688,7 +756,7 @@ build_jobtree (	schedwi_date jobset_date,
 	/* Build the job/jobset tree */
 	*nb_jobs = 0;
 	workload_date = schedwi_date_to_int (jobset_date);
-	ret = build_jobtree_recur (	NULL, "0", jobset_date,
+	ret = build_jobtree_recur (	NULL, "0", "", 0, jobset_date,
 					workload_date, "0", calendars,
 					hierarchy_list,
 					top, nb_jobs);
@@ -1042,8 +1110,8 @@ do_transition (	schedwi_date workload_date,
 /*
  * Change the status of a job/jobset. The fast parameter specifies if the
  * status must be propagated to the other jobs (0) or not (1).
- * This flag is used to avoid dead locks with clients and is used when a client
- * has sent the result status of a job.
+ * This flag is used to avoid dead locks with clients and is used when a
+ * client has sent the result status of a job.
  *
  * Return:
  *   0 --> No error
@@ -1060,10 +1128,10 @@ schedwi_jobtree_node_set_status (	schedwi_date workload_date,
 					char fast,
 					const char *status_message)
 {
-	int ret;
+	int ret, d;
 	schedwi_jobtree_link_ptr link;
 	schedwi_jobtree_node_ptr child, sibling;
-	job_status_state new_parent_status;
+	job_status_state new_parent_status, old_status;
 
 	if (ptr == NULL) {
 		return 0;
@@ -1074,18 +1142,36 @@ schedwi_jobtree_node_set_status (	schedwi_date workload_date,
 		return 0;
 	}
 
+	/* Check the number of retries if the job has failed */
+	if (	   ptr->node_type == 1
+		&& status == JOB_STATUS_STATE_FAILED
+		&& ptr->retries > ptr->retry_num)
+	{
+		ptr->retry_num++;
+		status = JOB_STATUS_STATE_WAITING;
+		ptr->run_time = schedwi_time_add_seconds (current_time,
+							ptr->retries_interval);
+	}
+
 	/* Update the status in the database */
-	ret = job_status_set (ptr->id, workload_date, status, current_time,
+	ret = job_status_set (ptr->id, ptr->path, workload_date, status,
+			current_time,
 			(duration >= 0)	? duration
 					: current_time - ptr->start_time,
+			ptr->retry_num,
 			status_message);
 	if (ret != 0) {
 		return ret;
 	}
 
 	/* Change the status of the job/jobset */
+	old_status = ptr->status;
 	ptr->status = status;
 
+	/* Call the `schedwi_status' function of the modules */
+	d = schedwi_date_to_int (workload_date);
+	module_status (d, ptr, old_status, status);
+
 	/* Set the start time */
 	if (status == JOB_STATUS_STATE_RUNNING) {
 		ptr->start_time = current_time;
@@ -1094,7 +1180,12 @@ schedwi_jobtree_node_set_status (	schedwi_date workload_date,
 	if (fast == 0) {
 		/* Tell all the jobs/jobsets waiting to check their status */
 		for (	link = ptr->links_in;
-			link != NULL && link->required_status == status;
+			   link != NULL
+			&& (   link->required_status == ptr->status
+			    || (link->required_status ==
+					JOB_STATUS_STATE_COMPLETED_OR_FAILED
+				&& ( ptr->status == JOB_STATUS_STATE_COMPLETED
+				  || ptr->status == JOB_STATUS_STATE_FAILED)));
 			link = link->next)
 		{
 			ret = schedwi_jobtree_node_check (workload_date,
@@ -1106,7 +1197,7 @@ schedwi_jobtree_node_set_status (	schedwi_date workload_date,
 		}
 
 		/* Start all the children */
-		if (status == JOB_STATUS_STATE_RUNNING) {
+		if (ptr->status == JOB_STATUS_STATE_RUNNING) {
 			for (	child = ptr->children;
 				child != NULL;
 				child = child->next)
@@ -1122,7 +1213,7 @@ schedwi_jobtree_node_set_status (	schedwi_date workload_date,
 	}
 
 	/* Update the parent jobset */
-	if (status == JOB_STATUS_STATE_RUNNING) {
+	if (ptr->status == JOB_STATUS_STATE_RUNNING) {
 		ret =  schedwi_jobtree_node_set_status (
 						workload_date, ptr->parent,
 						JOB_STATUS_STATE_RUNNING,
@@ -1132,10 +1223,10 @@ schedwi_jobtree_node_set_status (	schedwi_date workload_date,
 		}
 	}
 	else {
-		if (	   status == JOB_STATUS_STATE_COMPLETED
-			|| status == JOB_STATUS_STATE_FAILED)
+		if (	   ptr->status == JOB_STATUS_STATE_COMPLETED
+			|| ptr->status == JOB_STATUS_STATE_FAILED)
 		{
-			new_parent_status = status;
+			new_parent_status = ptr->status;
 			/* Check if all the next nodes are finished */
 			for (	sibling = ptr->next;
 				sibling != NULL;
@@ -1229,8 +1320,8 @@ schedwi_jobtree_node_check (	schedwi_date workload_date,
 			{
 				/* Too long - Stop the job */
 				lwc_writeLog (	LOG_INFO,
-	_("Workload %d: Job/Jobset Id %lld: running for too long. Stopping..."),
-						d, ptr->id);
+	_("Workload %d: %s (id %lld): running for too long. Stopping..."),
+						d, ptr->path, ptr->id);
 				reply = NULL;
 				ret = stopjob (d, ptr->id, &reply);
 				if (ret == 0) {
@@ -1242,13 +1333,14 @@ schedwi_jobtree_node_check (	schedwi_date workload_date,
 				if (ret > 0) {
 					if (reply != NULL) {
 						lwc_writeLog (LOG_ERR,
-		_("Workload %d: Job/Jobset Id %lld: failed to stop: %s"),
-							d, ptr->id, reply);
+		_("Workload %d: %s (id %lld): failed to stop: %s"),
+							d, ptr->path,
+							ptr->id, reply);
 					}
 					else {
 						lwc_writeLog (LOG_ERR,
-			_("Workload %d: Job/Jobset Id %lld: failed to stop"),
-							d, ptr->id);
+			_("Workload %d: %s (id %lld): failed to stop"),
+							d, ptr->path, ptr->id);
 					}
 				}
 				if (reply != NULL) {
@@ -1316,8 +1408,15 @@ schedwi_jobtree_node_check (	schedwi_date workload_date,
 
 			/* Check that all the links are okay */
 			for (	link = ptr->links;
-				link != NULL && link->linked_node->status ==
-							link->required_status;
+				   link != NULL
+				&& (link->linked_node->status ==
+							link->required_status
+					|| (link->required_status ==
+					JOB_STATUS_STATE_COMPLETED_OR_FAILED
+					   && (link->linked_node->status ==
+						JOB_STATUS_STATE_COMPLETED
+					       || link->linked_node->status ==
+						JOB_STATUS_STATE_FAILED)));
 				link = link->next);
 			if (link != NULL) {
 				/* Some links are not satisfied */
@@ -1342,8 +1441,8 @@ if(ptr->constraint_files != NULL) {
 /* No need to log a message if the remote file is missing
 			if (reply[0] != '\0') {
 				lwc_writeLog (LOG_INFO,
-	_("Workload %d: Job/Jobset Id %lld: Host Id %lld: check file %s: %s"),
-					d, ptr->id,
+	_("Workload %d: %s (id %lld): Host Id %lld: check file %s: %s"),
+					d, ptr->path, ptr->id,
 					(ptr->constraint_files)[i].host_id,
 					(ptr->constraint_files)[i].file,
 					reply);
@@ -1359,7 +1458,10 @@ if(ptr->constraint_files != NULL) {
 	}
 }
 
-			/* TODO: This a place to add other checks (modules?) */
+			/* Call the `schedwi_check' function of the modules */
+			if (module_check (d, ptr) != 0) {
+				return 0;
+			}
 
 			/* Start the job/jobset */
 			return schedwi_jobtree_node_set_status (
@@ -1539,9 +1641,10 @@ schedwi_jobtree_print_recur (schedwi_jobtree_node_ptr ptr, unsigned int depth)
 		spaces[space_len] = '\0';
 	}
 
-	lwc_writeLog (	LOG_INFO, "%s%s %lld %s",
+	lwc_writeLog (	LOG_INFO, "%s%s: %s (id %lld): %s",
 			(spaces != NULL) ? spaces : " ",
 			(ptr->node_type == 0) ? _("Jobset"): _("Job"),
+			ptr->path,
 			ptr->id,
 			job_status_state2str (ptr->status));
 
