@@ -1,19 +1,20 @@
 /* Schedwi
    Copyright (C) 2007 Herve Quatremain
 
-   This program is free software; you can redistribute it and/or modify
+   This file is part of Schedwi.
+
+   Schedwi is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
+   the Free Software Foundation; either version 3 of the License, or
    (at your option) any later version.
 
-   This program is distributed in the hope that it will be useful,
+   Schedwi is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU Library General Public License for more details.
+   GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 /* schedwisrv.c -- Schedwi server */
@@ -79,6 +80,7 @@ extern int errno;
 #include <signal_utils.h>
 #include <thread_init.h>
 #include <commands_thread.h>
+#include <module.h>
 #include <lib_functions.h>
 
 
@@ -378,6 +380,7 @@ main (int argc, char **argv)
 	const char *log_file, *date_format;
 	const char *iface, *server_port;
 	const char *server_key, *server_crt;
+	char quick_random;
 	const char *prog_name;
 	long int facility;
 	char test, foreground;
@@ -456,6 +459,13 @@ main (int argc, char **argv)
 		printf (_("Configuration file: %s\n"), configuration_file);
 	}
 
+	/*
+	 * Initialize the module engine
+	 */
+	if (module_init () != 0) {
+		return 1;
+	}
+
 	/*
 	 * Read the configuration file
 	 */
@@ -470,6 +480,7 @@ main (int argc, char **argv)
 			break;
 	}
 	if (ret != 0) {
+		module_exit ();
 		return 1;
 	}
 
@@ -480,12 +491,14 @@ main (int argc, char **argv)
 	ret = conf_get_param_string ("SSLServerCertificateFile", &server_crt);
 	ret += conf_get_param_string (	"SSLServerCertificateKeyFile",
 					&server_key);
+	ret += conf_get_param_bool ("SSLQuickRandom", &quick_random);
 #if HAVE_ASSERT_H
 	assert (ret == 0);
 #endif
 
-	if (net_init (server_crt, server_key) != 0) {
+	if (net_init (server_crt, server_key, quick_random) != 0) {
 		conf_destroy_srv ();
+		module_exit ();
 		return 1;
 	}
 
@@ -511,6 +524,7 @@ main (int argc, char **argv)
 		}
 		conf_destroy_srv ();
 		net_destroy ();
+		module_exit ();
 		return 1;
 	}
 
@@ -528,6 +542,7 @@ main (int argc, char **argv)
 		}
 		conf_destroy_srv ();
 		net_destroy ();
+		module_exit ();
 		return 1;
 	}
 
@@ -549,11 +564,13 @@ main (int argc, char **argv)
 			}
 			fputc ('\n', stderr);
 			conf_destroy_srv ();
+			module_exit ();
 			return 1;
 		}
 		end_mysql ();
 
 		conf_destroy_srv ();
+		module_exit ();
 		fputs (_("Fine."), stdout);
 		fputc ('\n', stdout);
 		return 0;
@@ -567,6 +584,7 @@ main (int argc, char **argv)
 			perror ("fork");
 			conf_destroy_srv ();
 			net_destroy ();
+			module_exit ();
 			return 1;
 		}
 	}
@@ -621,6 +639,7 @@ main (int argc, char **argv)
 		lwc_closeLog ();
 		conf_destroy_srv ();
 		net_destroy ();
+		module_exit ();
 		return 1;
 	}
 
@@ -645,6 +664,7 @@ main (int argc, char **argv)
 		close_pid_file (pid_file);
 		conf_destroy_srv ();
 		net_destroy ();
+		module_exit ();
 		return 1;
 	}
 
@@ -660,6 +680,7 @@ main (int argc, char **argv)
 		end_mysql ();
 		conf_destroy_srv ();
 		net_destroy ();
+		module_exit ();
 		return 1;
 	}
 	lwc_writeLog (LOG_INFO, _("Done"));
@@ -681,6 +702,7 @@ main (int argc, char **argv)
 		end_mysql ();
 		conf_destroy_srv ();
 		net_destroy ();
+		module_exit ();
 		return 1;
 	}
 
@@ -703,6 +725,7 @@ main (int argc, char **argv)
 		net_close_sock (sock);
 		conf_destroy_srv ();
 		net_destroy ();
+		module_exit ();
 		return 1;
 	}
 
@@ -714,6 +737,7 @@ main (int argc, char **argv)
 		load_workload_class_destroy (workload_list);
 		conf_destroy_srv ();
 		net_destroy ();
+		module_exit ();
 		return 1;
 	}
 
@@ -732,6 +756,7 @@ main (int argc, char **argv)
 		load_workload_class_destroy (workload_list);
 		conf_destroy_srv ();
 		net_destroy ();
+		module_exit ();
 		return 1;
 	}
 
@@ -746,6 +771,7 @@ main (int argc, char **argv)
 		load_workload_class_destroy (workload_list);
 		conf_destroy_srv ();
 		net_destroy ();
+		module_exit ();
 		return 1;
 	}
 
@@ -761,6 +787,7 @@ main (int argc, char **argv)
 		load_workload_class_destroy (workload_list);
 		conf_destroy_srv ();
 		net_destroy ();
+		module_exit ();
 		return 1;
 	}
 
@@ -785,6 +812,7 @@ main (int argc, char **argv)
 		load_workload_class_destroy (workload_list);
 		conf_destroy_srv ();
 		net_destroy ();
+		module_exit ();
 		return 1;
 	}
 
@@ -813,6 +841,7 @@ main (int argc, char **argv)
 			load_workload_class_destroy (workload_list);
 			conf_destroy_srv ();
 			net_destroy ();
+			module_exit ();
 			return 1;
 		}
 		switch (sig) {
@@ -850,6 +879,9 @@ main (int argc, char **argv)
 				/* Clean the network layer */
 				net_destroy ();
 
+				/* Destroy the module engine */
+				module_exit ();
+
 				/* Close the log file */
 				lwc_writeLog (LOG_INFO, _("Exiting"));
 				lwc_closeLog ();
