@@ -1,20 +1,20 @@
 /* Schedwi
    Copyright (C) 2007 Herve Quatremain
 
-   This program is free software; you can redistribute it and/or modify
+   This file is part of Schedwi.
+
+   Schedwi is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
+   the Free Software Foundation; either version 3 of the License, or
    (at your option) any later version.
 
-   This program is distributed in the hope that it will be useful,
+   Schedwi is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU Library General Public License for more details.
+   GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
-   USA.
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 /* sql_common.c -- Useful MYSQL functions */
@@ -74,6 +74,7 @@ init_db_connection (char **err_msg)
 	const char *host, *user, *passwd, *db, *unix_socket;
 	char *msg;
 	long int port;
+	my_bool reconnect = 1;
 
 	/* Get the connection parameter from the configuration file */
 	host = user = passwd = db = unix_socket = NULL;
@@ -98,6 +99,7 @@ init_db_connection (char **err_msg)
 	}
 
 	mysql_options (sql, MYSQL_READ_DEFAULT_GROUP, PACKAGE);
+	mysql_options (sql, MYSQL_OPT_RECONNECT, &reconnect);
 
 	if (mysql_real_connect (sql, host, user, passwd, db,
 				(unsigned int)port, unix_socket, 0) == NULL)
@@ -108,6 +110,9 @@ init_db_connection (char **err_msg)
 		sql = NULL;
 		return -1;
 	}
+
+	/* For MySQL prior to 5.0.19 */
+	mysql_options (sql, MYSQL_OPT_RECONNECT, &reconnect);
 	return 0;
 }
 
